﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EcrHost_CS.CommonCadWrapper;
using System.Runtime.InteropServices;

namespace EcrHost_CS
{
    class CommonCADWrapper
    {
// public members
        public delegate int prnCb(string text, int codepage, int imd);
        public delegate int dlgCb();
// private members
        private prnCb ecrPrinter = null;
        private dlgCb ecrDlgSignature = null;

        public CommonCADWrapper()
        {            
        }

        public void closeIface()
        {
            EcrHost_CS.CommonCadWrapper.GpePosLib.CloseLibrary();
        }

        //*********************************************************************************
        //
        //*********************************************************************************
        public int loadIface()
        {
            EcrHost_CS.CommonCadWrapper.GpePosLib.OpenLibrary("./EcrCfg.ini");
            int ver = EcrHost_CS.CommonCadWrapper.GpePosLib.GetVersion();
            return ver;
        }        

        private int textPrinterRutine(string text, int codepage, int imd, int test)
        {
            if (test == 0)
                // return (ecrPrinter.Invoke(text, codepage, imd));     //PP 24/08/2018
                return (ecrPrinter(text, codepage, imd));
            else
                return (0);
        }

        private int generalDialogRutine(int dlgIdx, int test)
        {
            int ret = -1;

            if (test == 0)
            {
                if (dlgIdx == GpeDialogIdx.DlgSignature)
                    ret = ecrDlgSignature(); //   ret = ecrDlgSignature.Invoke();    //PP 24/08/2018
            }
            else
                ret = 0;
            return (ret);
        }

        public int regPrinter(prnCb inPrinter)
        {
            ecrPrinter = inPrinter;            
            return (GpePosLib.RegisterPrintCB(textPrinterRutine));
        }

        public int regDialog(dlgCb inDialog)
        {
            ecrDlgSignature = inDialog;
            return (GpePosLib.RegisterDialogCB(generalDialogRutine));
        }

        public PaymentResponse posPayment(Int32 amount, Int32 amountCashBack, Int32 currency, String invoiceNr)
        {
            PaymentResponse payResp = null;
            int rootId = GpePosLib.CreatePrmTree(GpeTrxClass.GPE_TRX_PURCHASE_CB_CID);
            // check tree id                
            EmptyTrx.checkPositive(rootId, "Creation Root Tree Failed\n");
            PaymentRequest payReq = new PaymentRequest(rootId, amount, amountCashBack, currency, invoiceNr);
            EmptyTrx.checkNull(GpePosLib.RunTransaction(GpeTrxClass.GPE_TRX_PURCHASE_CB_CID, rootId), "Payment: transaction process failed\n");
            payResp = new PaymentResponse(rootId);
            return (payResp);
        }

       
        public PaymentResponse posPayment(Int32 amount, Int32 currency, String invoiceNr)
        {
            int ret = 0;
            PaymentResponse payResp = null;
            int rootId = GpePosLib.CreatePrmTree(GpeTrxClass.GPE_TRX_PURCHASE_CID);
            // check tree id
            EmptyTrx.checkPositive(rootId, "Creation Root Tree Failed\n");
            PaymentRequest payReq = new PaymentRequest(rootId, amount, currency, invoiceNr);
            ret = GpePosLib.RunTransaction(GpeTrxClass.GPE_TRX_PURCHASE_CID, rootId);
            EmptyTrx.checkNull(ret, "Payment: transaction process failed\n");
            payResp = new PaymentResponse(rootId);            
            return (payResp);
        }

        public RefundResponse posRefund(Int32 amount, Int32 currency, String invoiceNr)
        {
            RefundResponse refResp = null;
            int rootId = GpePosLib.CreatePrmTree(GpeTrxClass.GPE_TRX_REFUND_CID);
            // check tree id                
            EmptyTrx.checkPositive(rootId, "Creation Root Tree Failed\n");
            RefundRequest payReq = new RefundRequest(rootId, amount, currency, invoiceNr);
            EmptyTrx.checkNull(GpePosLib.RunTransaction(GpeTrxClass.GPE_TRX_REFUND_CID, rootId), "Refund: transaction process failed\n");
            refResp = new RefundResponse(rootId);
            return (refResp);
        }

        public SubTotalsResponse posSubTotals()
        {
            int rootId = GpePosLib.CreatePrmTree(GpeTrxClass.GPE_TRX_REFUND_CID);
            // check tree id                
            EmptyTrx.checkPositive(rootId, "Creation Root Tree Failed\n");
            SubTotalsRequest subTotReq = new SubTotalsRequest(rootId);
            EmptyTrx.checkNull(GpePosLib.RunTransaction(GpeTrxClass.GPE_TRX_CLOSE_TOTALS, rootId), "CloseTotals: transaction process failed\n");
            SubTotalsResponse subTotResp = new SubTotalsResponse(rootId);
            return (subTotResp);
        }
                
        public CloseTotalsResponse posCloseTotals(CloseTotalsIn inSum)
        {            
            int rootId = GpePosLib.CreatePrmTree(GpeTrxClass.GPE_TRX_REFUND_CID);
            // check tree id                
            EmptyTrx.checkPositive(rootId, "Creation Root Tree Failed\n");
            CloseTotalsRequest ctReq = new CloseTotalsRequest(rootId, inSum);
            EmptyTrx.checkNull(GpePosLib.RunTransaction(GpeTrxClass.GPE_TRX_CLOSE_TOTALS, rootId), "CloseTotals: transaction process failed\n");
            CloseTotalsResponse ctResp = new CloseTotalsResponse(rootId);
            return (ctResp);
        }
    }
}
