﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace EcrHost_CS.CommonCadWrapper
{
    public class EmptyTrx
    {        
        // Transaction types
        protected int prmTree = 0;
        private int trxType = 0;
        private StringBuilder tid = new StringBuilder (8+1);
        private StringBuilder timestamp = new StringBuilder(12+1);
        private int respCode = 50;
        public const int EMPTY_IN = 0;
        public const int EMPTY_OUT = 1;

        public EmptyTrx(int rootId, int inEmpty, int inTrx)
        {
            trxType = inTrx;
            if (inEmpty == EMPTY_IN)            
                prmTree = GpePosLib.GetInputTree(rootId);            
            else            
                prmTree = GpePosLib.GetOutputTree(rootId);            
            
            checkNotNull(prmTree, "EmptyTrn:in/outParamTree not valid\n");        
        }            
        // data interface
        public int getTrxType()
        {
            return(trxType);
        }

        public StringBuilder getTimeStamp()
        {            
            checkNull(GpePosLib.GetStringTag(prmTree, GpeRootTags.RTAG_TS, timestamp), "Payment: Resp.Data: Timestamp tag failed\n");            
            return (timestamp);
        }

        public StringBuilder getTid()
        {   
            checkNull(GpePosLib.GetStringTag(prmTree, GpeRootTags.RTAG_TID, tid), "Payment: Resp.Data: TID tag failed\n");
            return (tid);
        }

        public int getResponseCode()
        {            
            respCode = GpePosLib.GetIntTag(prmTree, GpeRootTags.RTAG_RESPCODE);
            checkPositive(respCode, "Payment: Resp.Data: TID tag failed\n");
            return (respCode);
        }

        override public string ToString()
        {
            string emptyTrxData = "RespCode: " + getResponseCode() + "\n";
                   emptyTrxData += "TimeStamp: "+getTimeStamp()+"\n";       
                   emptyTrxData += "TID: "+getTid()+"\n";
            return(emptyTrxData);
        }

        public bool isTrxApporoved()
        {
            if (respCode <= 10 && respCode >= 0)
                return (true);
            else
                return (false);
        }

        public string trxResultString()
        {
            if (isTrxApporoved())
                return ("APPROVED");
            else
                return ("DECLINED");
        }

        public static void checkNull(int inParam, string message)
        {
            if (inParam != 0)
                throw new CommonCADWrapperEx(message + ": " + inParam);
        }

        // If inParam != 0 is OK , expected values is not 0
        public static void checkNotNull(int inParam, string message)
        {
            if (inParam == 0)
                throw new CommonCADWrapperEx(message + ": " + inParam);
        }

        public static void checkPositive(int inParam, string message)
        {
            if (inParam < 0)
                throw new CommonCADWrapperEx(message + ": " + inParam);
        }
    }
}
