﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using EcrHost_CS.CommonCadWrapper;

namespace EcrHost_CS
{
    public partial class Frm_EcrHost : Form
    {
        private CommonCADWrapper ei = null;
        private TrxSummary trxSum = new TrxSummary();

        public Frm_EcrHost()
        {
            InitializeComponent();
        }

        //update summary
        private void updatePaymentSum(int amt)
        {
            trxSum.incCountDebit();
            trxSum.addDebitSum(amt);            
        }

        private void updatePaymentCashBackSum(int amt, int cbAmt)
        {
            updatePaymentSum(amt);
            trxSum.incCountCashBack();
            trxSum.addCashBackSum(cbAmt);
        }

        private void updateRefund(int amt)
        {
            trxSum.incCountCredit();
            trxSum.addCreditSum(amt);
        }
        
        private void refreshSummary()
        {
            Lb_DebitCnt.Text = trxSum.getTotalsDebitCount().ToString();
            Lb_DebitSum.Text = trxSum.getTotalsDebitSum().ToString();
            Lb_CreditCnt.Text = trxSum.getTotalsCreditCount().ToString();
            Lb_CreditSum.Text = trxSum.getTotalsCreditSum().ToString();
            Lb_CbCnt.Text = trxSum.getTotalsCBCount().ToString();
            Lb_CbSum.Text = trxSum.getTotalsCBSum().ToString();
            this.Invalidate();
        }

        private void nullSummary()
        {
            trxSum.Clear();
            refreshSummary();
        }

        private void newTransation(object btSender)
        {
            Button btn = (Button)btSender;
            string trxName = btn.Text;
            
            Lb_TrxResult.ForeColor = Color.Black;

            Rtb_Logger.AppendText("\n\n********* START ***********\n");
            Rtb_Logger.AppendText("         " + trxName + "            \n");
            Rtb_Logger.AppendText("*******************************\n");
            
            Lb_TrxName.Text = trxName;
            Lb_TrxResult.Text = "is pending";
            Rtb_Logger.Refresh();
            Lb_TrxResult.Refresh();
            Lb_TrxName.Refresh();
        }

        private void Bt_LoadLibrary_Click(object sender, EventArgs e)
        {
            Rtb_Logger.AppendText("Library is loading...\n");            
            ei = new CommonCADWrapper();
            if (ei != null)
            {
                int ver = ei.loadIface();
                Rtb_Logger.AppendText("Library ver.:" + (ver >> 24) + "." + ((ver >> 16)&0x00FF0000) + "." + ((ver>>8)&0x0000FF00) + "." + (ver&0x000000FF)+"\n");
            }
            else
                Rtb_Logger.AppendText("Library loading failed\n");

            CommonCADWrapper.prnCb dlgPrinter = new CommonCADWrapper.prnCb(formPrinter);
            if(ei.regPrinter(dlgPrinter) == 0)
                Rtb_Logger.AppendText("Printer registerd OK\n");
            else
                Rtb_Logger.AppendText("Printer registerd Failed\n");

            CommonCADWrapper.dlgCb dlgDialog = new CommonCADWrapper.dlgCb(sigantureDialog);
            if (ei.regDialog(dlgDialog) == 0)
                Rtb_Logger.AppendText("Dialog registerd OK\n");
            else
                Rtb_Logger.AppendText("Dialog registerd Failed\n");
        }

        private int formPrinter(string text, int codepage, int imd)
        {
            if (text.Length > 0)
            {
                if (imd == 1)
                    Rtb_Printer.AppendText("---------- [Immediate] ----------\n");
                else
                    Rtb_Printer.AppendText("------------ [Normal] -----------\n");
                Rtb_Printer.AppendText(text);
                Rtb_Printer.AppendText("--------------------------------------\n");

                Rtb_Printer.AppendText("\n");
                Rtb_Printer.AppendText("\n");
            }
            else
                Rtb_Printer.AppendText("No Ticket\n");

            return (0);
        }

        private int sigantureDialog()
        {
            int result = 0;            
            SignForm sg = new SignForm();            
            sg.ShowDialog();
            string test = sg.DialogResult.ToString();
            if (sg.DialogResult == DialogResult.No)
            {
                result = 1;
                Rtb_Logger.AppendText("Signature No\n");
            }
            else
                Rtb_Logger.AppendText("Signature Yes\n");
            
            Rtb_Logger.Refresh();
            return (result);
        }



        private void label4_Click(object sender, EventArgs e)
        {

        }

        private void Frm_EcrHost_Load(object sender, EventArgs e)
        {
            if (ei != null)
                ei.closeIface();
        }

        private void displayTrxResult(EmptyTrx trxResult)
        {
            if (trxResult.isTrxApporoved())                            
                Lb_TrxResult.ForeColor = Color.Green;            
            else           
                Lb_TrxResult.ForeColor = Color.Red;
            
            Rtb_Logger.AppendText(trxResult.ToString());
            Lb_TrxResult.Text = trxResult.trxResultString();
            Lb_TrxResult.Refresh();
        }

        private void displayTrxFailed()
        {
            Lb_TrxResult.ForeColor = Color.OrangeRed;
            Lb_TrxResult.Text = "FAILED!";            
        }

        private void Bt_Payment_Click(object sender, EventArgs e)
        {            
            try
            {
                Int32 amt = 0;
                Int32 curr = 0;
                String invNr = null;

                newTransation(sender);

                if(Tb_Amount.Text.Length > 0)
                    amt = Int32.Parse(Tb_Amount.Text);
                
                if(Tb_Currency.Text.Length > 0)
                    curr = Int32.Parse(Tb_Currency.Text);
         
       
                if(Tb_InvoiceNumber.Text.Length > 0)
                    invNr = Tb_InvoiceNumber.Text;

                PaymentResponse payRet = ei.posPayment(amt, curr, invNr);
                if (payRet != null)
                {                    
                    displayTrxResult(payRet);
                    if(payRet.isTrxApporoved())
                        updatePaymentSum(amt);
                }
                else
                {                 
                    throw new CommonCADWrapperEx("Transaction failed\n");
                }
            }
            catch (CommonCADWrapperEx ex)
            {
                Rtb_Logger.AppendText("Transaction Error: ex:" + ex.Message + "\n");
                displayTrxFailed();
            }
            catch (Exception genEx)
            {
                Rtb_Logger.AppendText("Other Error: ex:" + genEx.Message + "\n");
                displayTrxFailed();
            }
            finally
            {
                Rtb_Logger.AppendText("********** END ************\n");
                refreshSummary();
            }
        }

        private void Lb_TrxResult_Click(object sender, EventArgs e)
        {

        }

        private void Bt_Clear_Click(object sender, EventArgs e)
        {            
            Tb_Amount.Clear();
            Tb_AmountCB.Clear();
            Tb_Currency.Clear();
            Tb_InvoiceNumber.Clear();            
        }

        private void Bt_ClearLog_Click(object sender, EventArgs e)
        {
            Rtb_Logger.Clear();
        }

        private void Bt_ClearPrinter_Click(object sender, EventArgs e)
        {
            Rtb_Printer.Clear();
        }

        private void Bt_PaymentWithCB_Click(object sender, EventArgs e)
        {
            try
            {
                Int32 amt = 0;
                Int32 cbAmt = 0;
                Int32 curr = 0;
                String invNr = null;

                newTransation(sender);

                if (Tb_Amount.Text.Length > 0)
                    amt = Int32.Parse(Tb_Amount.Text);

                if (Tb_AmountCB.Text.Length > 0)
                    cbAmt = Int32.Parse(Tb_AmountCB.Text);

                if (Tb_Currency.Text.Length > 0)
                    curr = Int32.Parse(Tb_Currency.Text);

                if (Tb_InvoiceNumber.Text.Length > 0)
                    invNr = Tb_InvoiceNumber.Text;                

                PaymentResponse payCbRet = ei.posPayment(amt, cbAmt, curr, invNr);
                if (payCbRet != null)
                {                 
                    displayTrxResult(payCbRet);
                    if (payCbRet.isTrxApporoved())
                        updatePaymentCashBackSum(amt, cbAmt);
                }
                else
                {                    
                    throw new CommonCADWrapperEx("Transaction failed\n");
                }
            }
            catch (CommonCADWrapperEx ex)
            {
                Rtb_Logger.AppendText("Transaction Error: ex:" + ex.Message + "\n");
                displayTrxFailed();
            }
            catch (Exception genEx)
            {
                Rtb_Logger.AppendText("Other Error: ex:" + genEx.Message + "\n");
                displayTrxFailed();
            }
            finally
            {
                Rtb_Logger.AppendText("********** END ************\n");
                refreshSummary();
            }            
        }

        private void Bt_Refund_Click(object sender, EventArgs e)
        {
            try
            {
                Int32 amt = 0;
                Int32 curr = 0;
                String invNr = null;                

                newTransation(sender);

                if (Tb_Amount.Text.Length > 0)
                    amt = Int32.Parse(Tb_Amount.Text);

                if (Tb_Currency.Text.Length > 0)
                    curr = Int32.Parse(Tb_Currency.Text);

                if (Tb_InvoiceNumber.Text.Length > 0)
                    invNr = Tb_InvoiceNumber.Text;

                RefundResponse refRet = ei.posRefund(amt, curr, invNr);
                if (refRet != null)
                {
                    displayTrxResult(refRet);
                    if (refRet.isTrxApporoved())
                        updateRefund(amt);
                }
                else
                {
                    displayTrxFailed();                    
                    throw new CommonCADWrapperEx("Transaction failed\n");
                }
            }
            catch (CommonCADWrapperEx ex)
            {
                Rtb_Logger.AppendText("Transaction Error: ex:" + ex.Message + "\n");
                displayTrxFailed();
            }
            catch (Exception genEx)
            {
                Rtb_Logger.AppendText("Other Error: ex:" + genEx.Message + "\n");
                displayTrxFailed();
            }
            finally
            {
                Rtb_Logger.AppendText("********** END ************\n");
                refreshSummary();
            }
        }

        private void Bt_SubTotals_Click(object sender, EventArgs e)
        {
            try
            {
                newTransation(sender);             
                
                SubTotalsResponse ctResp = ei.posSubTotals();
                if (ctResp != null)                
                    displayTrxResult(ctResp);
            }
            catch (CommonCADWrapperEx ex)
            {
                Rtb_Logger.AppendText("Transaction Error: ex:" + ex.Message + "\n");
                displayTrxFailed();
            }
            catch (Exception genEx)
            {
                Rtb_Logger.AppendText("Other Error: ex:" + genEx.Message + "\n");
                displayTrxFailed();
            }
            finally
            {
                Rtb_Logger.AppendText("********** END ************\n");                
            }
        }

        private void Bt_CloseTotals_Click(object sender, EventArgs e)
        {
            try
            {
                Button btn = (Button)sender;
                string trxName = btn.Text;
                CloseTotalsIn ctSum;

                newTransation(sender);

                ctSum.debitCnt = trxSum.getTotalsDebitCount();
                ctSum.creditCnt = trxSum.getTotalsCreditCount();
                ctSum.cbCnt = trxSum.getTotalsCBCount();
                ctSum.debitSum = trxSum.getTotalsDebitSum();
                ctSum.creditSum = trxSum.getTotalsCreditSum();
                ctSum.cbSum = trxSum.getTotalsCBSum();
                CloseTotalsResponse ctResp = ei.posCloseTotals(ctSum);
                if (ctResp != null)
                {
                    Rtb_Logger.AppendText(trxName + " Finished\n");
                    displayTrxResult(ctResp);
                    nullSummary();
                }
                else
                {
                    displayTrxFailed();
                    throw new CommonCADWrapperEx("Transaction failed\n");
                }
            }
            catch (CommonCADWrapperEx ex)
            {
                Rtb_Logger.AppendText("Transaction Error: ex:" + ex.Message + "\n");
                displayTrxFailed();
            }
            catch (Exception genEx)
            {
                Rtb_Logger.AppendText("Other Error: ex:" + genEx.Message + "\n");
                displayTrxFailed();
            }
            finally
            {
                Rtb_Logger.AppendText("********** END ************\n");
                Rtb_Logger.Refresh();
                Lb_TrxResult.Refresh();
            }
        }
    }
}
