/*
    +-----------------------------------------------------------------------+
    |   File name :             Ecr_Test.cpp                                |
    |   Description :           Main test module for usage the DLL interface|
    |                                                                       |
    |                           Iterface betwwen the ECR test base and DLL  |
    |                           library DLL.                                |    
    |                                                                       |
    |   Creation date :         27-05-2009                                  |
    |                                                                       |
    |   @author:                Pavel Perman                                | 
    |   @version:               1.0 (developed)                             |
    |   Copyright :             GPE                                         |
    |                                                                       |
    +-----------------------------------------------------------------------+
*/

/* ****************************************************************** Includes */

#include "stdafx.h"
#include <ctime>
#include <iostream>
#include "windows.h"
#include "Logger.h"

#include "SystemApi.h"
#include "Ecr_Test.h"
#include "Totals.h"
#include "GpeEcrCommLib.h"


/* ***************************************************************** Macro def.       */
#define CHECK_NULL(condition, exText, exLbl)	if((condition) == NULL) {	printf("%s", exText); goto exLbl; }
#define CHECK_COND(condition, exText, exLbl)	if(!(condition)) {	printf("%s", exText); goto exLbl; }
#define CONF_FILE "c:\\ATM_Proj\\POS_ECR\\Conf_Files\\EcrCfg.ini"
/* ***************************************************************** Type def.        */

/* ***************************************************************** Public variable  */

/* ***************************************************************** Private variable */
const char *DLG_TITLE_SIGNATURE_CHECK = "The signature comparison";
const char * DLG_TEXT_SIGNATURE_CHECK =  \
"Please compare the signature on the receipt\n\
and on the card.  Are these signatures the same?\n\
Yes/No\n";

HANDLE evHnd = 0;
static HINSTANCE hDll;
static ecrIface_t fcEcr;
static int loadAmt = 0;
static int loadcbAmt = 0;
static int delay = 0;
static int loadTest = 0;

/* **************************************************************** Private functions */
static int dynamicLoadLibrary(ecrIface_t *pFc, char *pDllName);
static int dynamicFreeLibrary(void);
static int initEcrMain(ecrIface_t *pFc);
static void runEcrManager(ecrIface_t *pFc);
// transaction calls
static int callPurchase(ecrIface_t *pFc);
static int callPurchaseWithCashBack(ecrIface_t *pFc);
static int callRefund(ecrIface_t *pFc);
static int callVoid (ecrIface_t *pFc);
static int callSubTotals (ecrIface_t *pFc);
static int callCloseTotals (ecrIface_t *pFc);

static int rangedRand( int r_min, int range_max);
int showDialogs(const char *pTitleDialog, const char *pText );
static int getLoadAmount(void);
static int getLoadAmountCb(void);
static int getLoadDelayTime(void);
static void setLoadDelayTime(void);
static int isLoad(void);
static void setLoad(void);
static void loadTestDll(unsigned int cnt);

/**********************************************************************************************************/
/**
 * <p>Title:  DllMain</p>
 * <p>Description: Hlavn9 funkce spustena pri zavedeni DLL do pameti procesu.
 *                 Dale pak spoustena pokud hlavni proces spusti a ukonci vlakno. 
 *                 A pokud hlavni proces odstrani DLL z pameti.</p>
 *
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param:  Podle Win32 API
 * @return  TRUE:   proces byl uspesne nacten.
 *          FALSE:   - chyba 
 */
int main(int argc, char* argv[])
{
    printf("Start ECR interface\n");
    if(dynamicLoadLibrary(&fcEcr, ECRDLLNAME) == 0)
    {        
		if(fcEcr.GpePosLib_Open != NULL)
        {
			if(fcEcr.GpePosLib_Open("./EcrCfg.ini") == GPE_OK)		//PP 24/08/2018 - added conf.
            {
				if(initEcrMain(&fcEcr) == 0)
                {
                    printf("The ECR started\n");
                    runEcrManager(&fcEcr);
                }
                else
                    printf("InitEcrMain failed");
            }
            else            
                printf("PosLib Open Failed!\n");
		}
        else        
            printf("PosLib not correctlly loaded!\n");        
    }        
	if(fcEcr.GpePosLib_Close != NULL)
		fcEcr.GpePosLib_Close();
    
	dynamicFreeLibrary();
	DBG_CloseLogFile();    
	printf("\n");
    printf("The program will be terminated..\n");
    return 0;
}

/**********************************************************************************************************/
/**
 * <p>Title:  showDialogById</p>
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param: 
 * @return:
 */
int __GPESTDCALL showDialogById(int dlgId, int flgTst)
{
	int ret = GPE_UNEXPECED_ERR;
	if(!flgTst)	
	{
		switch(dlgId)
		{
			case GPE_DIALOG_IDX_SIGNATURE_REQUIRED:	ret = showDialogs(DLG_TITLE_SIGNATURE_CHECK, DLG_TEXT_SIGNATURE_CHECK); break;        
			default:								ret = GPE_INPUT_PARAMS_ERR;	break;
		}	
	}
	else
	{
		printf("Registr CB dialog test OK");	
		ret = GPE_OK;
	}
	return(ret);
}

int __GPESTDCALL printReceipt(char *pText, int isoId, int imd, int flgTst)
{
	int ret = GPE_UNEXPECED_ERR;
	if(!flgTst)		
	{		
		if(imd)		
			printf("\n\n------- Ticket [Immediated] ---------\n");				
		else
			printf("\n\n---------- Ticket [Normal] ----------\n");				
		
		printf("%s\n", pText);	
		printf("-------------------------------------\n\n");		
	}
	else
	{
		printf("Registr CB Printer test OK");	
		ret = GPE_OK;
	}
	return(ret);
}

int __GPESTDCALL dispalyText(char *pText, int isoId, int flgTst)
{
	int ret = GPE_UNEXPECED_ERR;
	if(!flgTst)		
	{
		
		printf("\n\n----------- DISPLAY -------------\n");	
		printf("%s\n", pText);	
		printf("---------------------------------\n\n");	
	}
	else
	{
		printf("Registr CB Display test OK");
		ret = GPE_OK;
	}
	return(ret);
}



/**********************************************************************************************************/
/**
 * <p>Title:  initEcrMain</p>
 * <p>Description: Funkce pro hlavni inicializaci pokladniho rozhranni</p>
 *
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param: 
 * @return:
 */
static int initEcrMain(ecrIface_t *pFc)
{
    // init dialogs
	srand(time(0));
	Totals_clear();
	pFc->GpePosLib_Dialog(showDialogById);	// register dialogs
	pFc->GpePosLib_Print(printReceipt);		// register printers
	pFc->GpePosLib_Display(dispalyText);
//	DBG_OpenLogFile("EcrLog.log");
    return(0);
}


/**********************************************************************************************************/
/**
 * <p>Title:  callPurchase</p>
 * <p>Description: Funkce vyzve k zadani vsech nutnych udaju pro transaci Prodej a tuto transakci spusti.</p>
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param: 
 * @return: ( 0) - trn. OK
 *          (!0) - trn. fail !
 */
static int callTransaction(ecrIface_t *pFc, int idTrx, int autoTest)
{        
    unsigned int amount = 0, cashBack = 0;    
    char timedate[15];
	char invoiceNo[10+1];
	int currency = 0;
    int retPurchase = 0;
	int ret = 0;
	int retTrn = 0;
	
    SysApi_Clrscr();	
	fflush(stdin);
    memset(timedate, 0, sizeof(timedate));
	memset(invoiceNo, 0, sizeof(invoiceNo));
	// timestamp
	SysApi_GetTimeStamp(NULL, timedate);
	printf("TimeStamp %s\n", timedate);		// is not used

    // input screen
	if(idTrx == GPE_TRX_PURCHASE_CID || idTrx == GPE_TRX_REFUND_CID || idTrx == GPE_TRX_PURCHASE_CB_CID)
	{					
		amount = getLoadAmount();
		if(!autoTest)
		{
			printf("Enter Amount:");
			scanf("%u", &amount);
			printf("Currency:");
			scanf("%u", &currency);
		}
		else
		{
			printf("Auto Amount:%d\n", amount);
			currency = 203;
			printf("Auto Currency:%d\n", currency);			
		}		
	}
	if(idTrx == GPE_TRX_PURCHASE_CB_CID)
	{
		cashBack = getLoadAmount();
		if(!autoTest)
		{
			printf("Enter CashBack:");
			scanf("%u", &cashBack);
		}
		else							
			printf("Auto CashBack:%d\n", cashBack );				
	}
	if(idTrx == GPE_TRX_PURCHASE_CID || idTrx == GPE_TRX_REFUND_CID)
	{	
		
		if(!autoTest)
		{
			printf("Invoice Number:");
			scanf("%s", invoiceNo);
		}
		else
		{			
			sprintf(invoiceNo, "%u", rangedRand(1, 1000000));
			printf("Auto Invoice Number: %s\n", invoiceNo);
		}
	}

	int rootId  = pFc->GpePosLib_CreatePrmTree(idTrx);
	if(rootId >= 0)
	{
		int inTreeId = pFc->GpePosLib_GetInputTree(RTAG_ROOT);		// demand
		int outTreeId = pFc->GpePosLib_GetOutputTree(RTAG_ROOT);	// result
		if(inTreeId > 0 && outTreeId > 0)
		{
			if(cashBack > 0)
				CHECK_COND((pFc->GpePosLib_AddIntTag(inTreeId, RTAG_CB_AMT, cashBack) == 0), "Tag Amount Err\n", LBL_TAG_ERR);
			if(amount > 0)
				CHECK_COND((pFc->GpePosLib_AddIntTag(inTreeId, RTAG_AMT, amount) == 0), "Tag Amount Err\n", LBL_TAG_ERR);
			if(strlen(invoiceNo) > 0)
				CHECK_COND((pFc->GpePosLib_AddStringTag(inTreeId, RTAG_INVOICE_NO, invoiceNo) == 0), "Tag InvoiceNo\n", LBL_TAG_ERR);
			if(currency > 0)
				CHECK_COND((pFc->GpePosLib_AddIntTag(inTreeId, RTAG_CUR, currency) == 0), "Tag Currency Err\n", LBL_TAG_ERR);
		
			if(idTrx == GPE_TRX_CLOSE_TOTALS)
			{
				int subTreeIn = pFc->GpePosLib_AddSubTree(inTreeId, RTAG_SUM);
				if(subTreeIn > 0)
				{
					char summary[50];
					CHECK_COND((pFc->GpePosLib_AddIntTag(subTreeIn, STAG_DEBIT_COUNT, Totals_getTotalsDebitCount()) == 0), "Tag Debit count Err\n", LBL_TAG_ERR);
					memset(summary, 0, sizeof(summary));
					Totals_getTotalsDebitSum(summary);
					CHECK_COND((pFc->GpePosLib_AddStringTag(subTreeIn, STAG_DEBIT_SUM, summary) == 0), "Tag Debit Sum Err\n", LBL_TAG_ERR);

					CHECK_COND((pFc->GpePosLib_AddIntTag(subTreeIn, STAG_CREDIT_COUNT, Totals_getTotalsCreditCount()) == 0), "Tag Credit count Err\n", LBL_TAG_ERR);
					memset(summary, 0, sizeof(summary));
					Totals_getTotalsCreditSum(summary);
					CHECK_COND((pFc->GpePosLib_AddStringTag(subTreeIn, STAG_CREDIT_SUM, summary) == 0), "Tag Credit Sum Err\n", LBL_TAG_ERR);

					CHECK_COND((pFc->GpePosLib_AddIntTag(subTreeIn, STAG_CB_COUNT, Totals_getTotalsCBCount()) == 0), "Tag CB count Err\n", LBL_TAG_ERR);
					memset(summary, 0, sizeof(summary));
					Totals_getTotalsCBSum(summary);
					CHECK_COND((pFc->GpePosLib_AddStringTag(subTreeIn, STAG_CB_SUM, summary) == 0), "Tag CB Sum Err\n", LBL_TAG_ERR);
				}
				else				
					goto LBL_TAG_ERR;				
			}

			retTrn = pFc->GpePosLib_Transaction(idTrx, rootId);
			if(retTrn == GPE_OK)
			{
				struct 
				{
					int respCode;
					char authCode[10];
					char pan[30];
					char card[20];
					char timestamp[15];
					char uniqueTrnId[15];
					char tid[10];
					char seqNum[11];
					char receipt[4096 + 1];
				}rsp;

				memset(&rsp, 0, sizeof(rsp));
				
				// Terminal ID - madatory
				if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_TID, rsp.tid) == 0)
					printf("TID[%s]\n", rsp.tid);
				else
					printf("No TID\n");

				// TimeStamp - mandatory
				if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_TS, rsp.timestamp) == 0)
					printf("TimeStamp[%s]\n", rsp.timestamp);
				else
					printf("No TimeStamp\n");
				
				// Response code - mandatory
				rsp.respCode = pFc->GpePosLib_GetIntTag(outTreeId, RTAG_RESPCODE);
				printf("RespCode[%d]\n", rsp.respCode);		
				if(rsp.respCode >= 0 && rsp.respCode <= 10)
				{
					// authorization code - mandatory
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_AUTH_CODE, rsp.authCode) == 0)
						printf("AuthCode[%s]\n", rsp.authCode);
					else
						printf("No AuthCode\n");
					
					// Pan - mandatory
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_PAN, rsp.pan) == 0)
						printf("PAN[%s]\n", rsp.pan);
					else
						printf("No PAN\n");
					
					// Card Name
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_CARD, rsp.card) == 0)
						printf("Card[%s]\n", rsp.card);
					else
						printf("No Card Name\n");
					
					// TrnId
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_TRXNO, rsp.uniqueTrnId) == 0)
						printf("TrxId[%s]\n", rsp.uniqueTrnId);
					else
						printf("No TrxId\n");

					// SeqNum
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_SQNO, rsp.seqNum) == 0)
						printf("SeqNum[%s]\n", rsp.seqNum);
					else
						printf("No SeqNum\n");

					// Receipt - merchant
					memset(rsp.receipt, 0, sizeof(rsp.receipt));
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_RECEIPT_1, rsp.receipt) == 0)
						printf("Receipt Merchant:%s\n", rsp.receipt);
					else
						printf("No Receipt Merchant\n");
					
					// Receipt - customer
					if(pFc->GpePosLib_GetStringTag(outTreeId, RTAG_RECEIPT_2, rsp.receipt) == 0)
						printf("Receipt Customer:%s\n", rsp.receipt);
					else
						printf("No Receipt Customer\n");
					
				
					printf("Transaction finished: APPROVED\n");
					if(idTrx == GPE_TRX_PURCHASE_CID || idTrx == GPE_TRX_PURCHASE_CB_CID)
					{
						Totals_incCountDebit();
						Totals_addDebitSum(amount+cashBack);
					}
					if(idTrx == GPE_TRX_REFUND_CID)
					{												
						Totals_incCountCredit();
						Totals_addCreditSum(amount);
					}
					if(idTrx == GPE_TRX_PURCHASE_CB_CID)
					{						
						Totals_incCountCashBack();
						Totals_addCBSum(cashBack);
					}
					if(idTrx == GPE_TRX_CLOSE_TOTALS)
					{
						Totals_print();
						Totals_clear();
					}
					if(idTrx == GPE_TRX_SUB_TOTALS)
					{
						Totals_print();
					}
				}
				else
					printf("Transaction finished: DECLINE\n");
			}
			else
				printf("The transaction processing failed - the response data are not valid [r%d]\n", retTrn);
		}
	}
if(!autoTest)
{	
	char    actKey = 0;
	printf("Press ESC to go back...");
	do
	{
		if (_kbhit())
			actKey = _getch();
		SysApi_Delay(100);				// wait for 100ms
	} while (actKey != 0x1B);
}
else
	SysApi_Delay(getLoadDelayTime()*1000);

LBL_TAG_ERR:
	ret = -1;
	goto LBL_FINISHED;

LBL_FINISHED:
	pFc->GpePosLib_ReleaseTree(rootId);
	return(retPurchase);           
}

/**********************************************************************************************************/
static int callPurchase(ecrIface_t *pFc, int autoTest)
{
	int ret = 0;
	ret = callTransaction(pFc, GPE_TRX_PURCHASE_CID, autoTest);
	return(ret);
}

/**********************************************************************************************************/
static int callPurchaseWithCashBack(ecrIface_t *pFc, int autoTest)
{
	int ret = 0;
		ret = callTransaction(pFc, GPE_TRX_PURCHASE_CB_CID, autoTest);
	return(ret);
}
/**********************************************************************************************************/
/**
 * <p>Title:  callRefund</p>
 * <p>Description: Funkce vyzve k zadani vsech nutnych udaju pro transaci Navrat a tuto transakci spusti.</p>
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param: 
 * @return: ( 0) - trn. OK
 *          (!0) - trn. fail !
 */
static int callRefund(ecrIface_t *pFc, int autoTest)
{
	int ret = 0;
		ret = callTransaction(pFc, GPE_TRX_REFUND_CID, autoTest);
	return(ret);    
}
/**********************************************************************************************************/
static int callVoid (ecrIface_t *pFc, int autoTest)
{
	int ret = 0;
		printf("The transaction type Void is not supported");
	return(ret);
}
/**********************************************************************************************************/
static int callSubTotals (ecrIface_t *pFc, int autoTest)
{	
	int ret = 0;
		ret = callTransaction(pFc, GPE_TRX_SUB_TOTALS, autoTest);
	return(ret);  
}
/**********************************************************************************************************/
static int callCloseTotals (ecrIface_t *pFc, int autoTest)
{
	int ret = 0;
		ret = callTransaction(pFc, GPE_TRX_CLOSE_TOTALS, autoTest);
	return(ret);  
}
/**********************************************************************************************************/

static int rangedRand( int r_min, int r_max)
{        
   int u = (int)rand()/(RAND_MAX + 1) * (r_max - r_min)+ r_min;
   return(u);
}

static void setLoad(void)
{
	loadTest = 1;
}

static int isLoad(void)
{
	return(loadTest);
}

static void setLoadAmount(int amt)
{
	loadAmt = amt;
}
static void setLoadAmountCb(int amtcb)
{
	loadcbAmt = amtcb;
}
static int getLoadAmount(void)
{
	return(loadAmt);
}

static int getLoadAmountCb(void)
{
	return(loadcbAmt);
}

static void setLoadDelayTime(void)
{
	printf("Enter delay between transactions:");
	scanf("%d", &delay);
}

static int getLoadDelayTime(void)
{
	return(delay);
}

static int callLoadTest (ecrIface_t *pFc)
{
	int ret = 0;
	unsigned int count = 100;
	int countTrx = 0;
	
	SysApi_Clrscr();
	printf("Enter count of transactions:");
	scanf("%u", &count);	
	setLoadDelayTime();

	
	for(unsigned int i=0; i<count; i++)		
	{
		int rAmt = rangedRand(100, 150000);			// 1-1500Kc		
		int rCBAmt = rangedRand(100, 150000);			// 1-1500Kc		
//		int type = rangedRand(1, 100);	// nahodny typ transakce						
		int type = 30; // payment
		
		if(type >= 10 && type <= 20)
			callCloseTotals(&fcEcr, 1);
		
		else if(type >= 80 && type <= 90)
			callSubTotals(&fcEcr, 1);

		else if(type < 10 )
		{			
			setLoadAmount(rAmt+30000);
			setLoadAmountCb(rCBAmt);				// CB je min. pro amt > 300 a amx. 1500 CB - proto max. castka omezena na 1500		
			callPurchaseWithCashBack(&fcEcr, 1);
		}

		else if(type >=91 && type <= 100)
		{
			setLoadAmount(rAmt);
			callRefund(&fcEcr, 1);
		}
		else
		{
			setLoadAmount(rAmt);
			callPurchase(&fcEcr, 1);
		}
	}

	setLoadAmount(0);
	setLoadAmountCb(0);
	return(ret);
}
/**********************************************************************************************************/
static void displayResult(int result)
{
	switch(result)
	{
		case 
			GPE_OK:	printf("GPE_OK\n");	
//			printCardDetail(pFc);            
		break;
		case GPE_INPUT_PARAMS_ERR:				printf("GPE_INPUT_PARAMS_ERR\n");		break;
		case GPE_LIB_NOT_FOUND_ERR:				printf("GPE_LIB_NOT_FOUND_ERR\n");		break;
		case GPE_GENERAL_ERR:					printf("GPE_GENERAL_ERR\n");			break;
		case GPE_APP_IFACE_NOT_INIT_ERR:		printf("GPE_APP_IFACE_NOT_INIT_ERR\n");	break;
		case GPE_APP_NO_APP_DATA_ERR:			printf("GPE_APP_NO_APP_DATA_ERR\n");	break;
		case GPE_APP_DATA_NOT_VALID_ERR:		printf("GPE_APP_DATA_NOT_VALID_ERR\n");	break;
		case GPE_APP_MANDATORY_DATA_ERR:		printf("GPE_APP_MANDATORY_DATA_ERR\n");	break;
		case GPE_TRX_PRC_INTERUPTED_ERR:		printf("GPE_TRX_PRC_INTERUPTED_ERR\n");	break;
		case GPE_TRX_IN_PROGRESS_ERR:			printf("GPE_TRX_IN_PROGRESS_ERR\n");	break;
		case GPE_COMM_TERMINAL_NOT_FOUND_ERR:	printf("GPE_COMM_TERMINAL_NOT_FOUND_ERR\n");	break;
		case GPE_COMM_TERMINAL_RESP_TO_ERR:		printf("GPE_COMM_TERMINAL_RESP_TO_ERR\n");		break;
		case GPE_COMM_TERMINAL_CORRUPTED_ERR:	printf("GPE_COMM_TERMINAL_CORRUPTED_ERR\n");	break;
		case GPE_UNEXPECED_ERR:					printf("GPE_UNEXPECED_ERR\n");			break;
		case GPE_INTERNAL_ERR:					printf("GPE_INTERNAL_ERR\n");			break;
	}		
}

/**********************************************************************************************************/
/**
 * <p>Title:  runEcrManager</p>
 * <p>Description: Hlavni funkce pro spousteni jednotlivych funkci ecr rozhranni. Sekundarne propagovane na POS.</p>
 *
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param:  (In) - struktura obsahujici vsechny funkce dostupnych v ECR rozhranni.  
*/
static void runEcrManager(ecrIface_t *pFc)
{
    int     retManager = TRUE;
	int		trnEnd = FALSE;
    char    actKey = 0;                              
    int     retPurchase = -1;    
	unsigned int ver = (unsigned int)pFc->GpePosLib_GetVersion();
	do
	{
        SysApi_Clrscr();
        printf("Welcomme to the ECR manager\n");
		printf("GPE ECR DLL ver.:%u.%u.%u.%u\n", (ver>>24)&0xff, (ver>>16)&0xff, (ver>>8)&0xff, ver&0xff);		
		printf("\n");
        printf("0: Purchase\n");
        printf("1: Purchase with cashBack\n");            
		printf("2: Refund\n");   
		printf("3: Void\n");   
		printf("4: SubTotals\n");
		printf("5: Close Totals\n");
		printf("-------------------\n");
		printf("6:LoadTest\n");
		printf("7:LoadTest Dll \n");
		printf("n: END\n");
        printf("\n");             

		do
		{
			actKey = _kbhit();		
			if(actKey)
			{
				actKey = _getch();		
				switch(actKey)
				{                                        
					case '0':                     
						displayResult(callPurchase(&fcEcr, 0));
						trnEnd = TRUE;
						break;

					case '1':                
						displayResult(callPurchaseWithCashBack(&fcEcr, 0));
						trnEnd = TRUE;
						break;
					
					case '2':                
						displayResult(callRefund(&fcEcr, 0));
						trnEnd = TRUE;
						break;

					case '3':                
						displayResult(callVoid(&fcEcr, 0));
						trnEnd = TRUE;
						break;
					
					case '4':
						displayResult(callSubTotals(&fcEcr, 0));
						trnEnd = TRUE;
						break;

					case '5':
						displayResult(callCloseTotals(&fcEcr, 0));                    					                    
						trnEnd = TRUE;
						break;

					case '6':
						setLoad();
						callLoadTest(&fcEcr);
						trnEnd = TRUE;
						break;
					case '7':
						{
							unsigned int cnt = 0;
							std::cout << "Start Load tests for DLL, entry numbe of cyklus : " << std::endl;
							std::cin >> cnt;
							setLoad();
							loadTestDll(cnt);
						}
						trnEnd = true;
						break;
					case 'n':    // esc                
					case 'N':    // esc                
						trnEnd = TRUE;
						retManager = FALSE;
						break;

					default: 
						break;                
				}
			}		
		}while(!trnEnd);
		trnEnd = FALSE;
	}while(retManager);
}

/**********************************************************************************************************/
/**
 * <p>Title:  showDialogs</p>
 * <p>Description: Funkce dynamicky nacte knihovnu do pameti procesu.</p>
 *
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param  loadTested - pro automaticki load testy , kde neny treba provadit kontrolu podpisu
 *              0 - standartny , 1 - utomaticky tety  
 * @return  TRUE:   Dialog was confirmed by key Y.
 *          FALSE:  Dialog wasn't confirmed by key N.
 */
int showDialogs(const char *pTitleDialog, const char *pText)        
//int __stdcall showDialogs(BSTR pTitleDialog, BSTR pText)        //PP VB
{
    int     retDialog = 0;
    int     lenText = 0;
    char    actKey = 0;
    char    paddingTitle[30], paddingText[30];

	if(isLoad())
		return(0);

    memset(paddingText,' ', sizeof(paddingText));
    memset(paddingTitle,' ', sizeof(paddingTitle));
    
    paddingTitle[30-6-strlen(pTitleDialog)] = 0;        
    paddingText[30-strlen(pText)] = 0;

    lenText = strlen(pText);

    printf("\n\n\n");  
	printf("+---------- Dialog ------------+\n");
    printf("Title %s%s\n", pTitleDialog, paddingTitle);
    printf("\n");
    printf("%s\n", pText);
    printf("+------------------------------+\n");
    printf("|        Yes = Y / No = N      |\n");
    printf("+------------------------------+\n");
    
	//DBG_Logger("GetResult Dialog\n");
    while(!loadTest)
    {        
        actKey = SysApi_WaitGetKey();

        if(actKey == 'N' || actKey == 'n')
        {
            retDialog = -1;
            printf("No\n");
            break;
        }
        
        if(actKey == 'Y' || actKey == 'y')
        {
            retDialog = 0;
            printf("Yes\n");
            break;
        }
        Sleep(200);
    }

	// automaticky tety
	if(loadTest != 0)
		retDialog = true;

    return(retDialog);
}

/**********************************************************************************************************/
/**
 * <p>Title:  dynamicLoadLibrary</p>
 * <p>Description: Funkce dynamicky nacte knihovnu do pameti procesu.</p>
 *
 * <p>Copyright: Copyright (c) 2009</p>
 * <p>Company: GPEurope</p>
 * @author Pavel PERMAN
 * @version 1.0
 * @Param:
 * @return  0:   proces byl uspesne nacten.
 *          -1:   - chyba 
*/

int dynamicLoadLibrary(ecrIface_t *pFc, char *pDllName)
{ 
    int retLoad = 0;

    if((hDll = LoadLibrary(pDllName)) == NULL)    
        printf("The library was not found %s\n", pDllName);            
    else
    {   		
		pFc->GpePosLib_Open			= (pFc_OpenLibrary) GetProcAddress(hDll, GPEPOSLIB_OPENLIBRARY);        
        pFc->GpePosLib_Close		= (pFc_CloseLibrary) GetProcAddress(hDll, GPEPOSLIB_CLOSELIBRARY);
		pFc->GpePosLib_GetVersion	= (pFc_GetVersion) GetProcAddress(hDll, GPEPOSLIB_GETVERSION);        

		pFc->GpePosLib_Transaction	= (pFc_Transaction) GetProcAddress (hDll, GPEPOSLIB_RUN_TRANSACTION);

		pFc->GpePosLib_Display		= (pFc_RegCB_Display) GetProcAddress(hDll, GPEPOSLIB_REGISTER_DISPLAYCB);
		pFc->GpePosLib_Print		= (pFc_RegCB_Print) GetProcAddress(hDll, GPEPOSLIB_REGISTER_PRINTCB);
		pFc->GpePosLib_Dialog		= (pFc_RegCB_Dialog) GetProcAddress(hDll, GPEPOSLIB_REGISTER_DIALOGCB);

		pFc->GpePosLib_CreatePrmTree= (pFc_CreatePrm) GetProcAddress(hDll, GPEPOSLIB_CREATE_PRM_TREE);
		pFc->GpePosLib_ReleaseTree	= (pFc_ReleasePrm)GetProcAddress(hDll, GPEPOSLIB_RELEASE_TREE);
		
		pFc->GpePosLib_GetInputTree	= (pFc_GetInputPrm)GetProcAddress(hDll, GPEPOSLIB_GET_INPUT_TREE);
		pFc->GpePosLib_GetOutputTree= (pFc_GetOutputPrm)GetProcAddress(hDll, GPEPOSLIB_GET_OUTPUT_TREE);
		
		pFc->GpePosLib_AddStringTag	= (pFc_AddStringTag)GetProcAddress(hDll, GPEPOSLIB_ADD_STRING_TAG);
		pFc->GpePosLib_AddIntTag	= (pFc_AddIntTag)GetProcAddress(hDll, GPEPOSLIB_ADD_INT_TAG);
		pFc->GpePosLib_AddSubTree	= (pFc_AddSubPrm)GetProcAddress(hDll, GPEPOSLIB_ADD_SUB_TREE);
		
		pFc->GpePosLib_GetStringTag	= (pFc_GetStringTag)GetProcAddress(hDll, GPEPOSLIB_GET_STRING_TAG);
		pFc->GpePosLib_GetIntTag	= (pFc_GetIntTag)GetProcAddress(hDll, GPEPOSLIB_GET_INT_TAG);
		pFc->GpePosLib_GetSubTree	= (pFc_GetSubPrmTag)GetProcAddress(hDll, GPEPOSLIB_GET_SUB_TREE);
	
        // inicializace odkazu na funkce z DLL                                                        
        CHECK_NULL(pFc->GpePosLib_Open,			"iFace Open is NULL\n",			EX_NULL_ERR);
        CHECK_NULL(pFc->GpePosLib_Close,		"iFace Close is NULL\n",		EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_GetVersion,	"iFace Version is NULL\n",		EX_NULL_ERR);
        CHECK_NULL(pFc->GpePosLib_Transaction,	"iFace Transaction is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_Display,		"iFace CB Display is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_Print,		"iFace CB Printer is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_Dialog,		"iFace CB Dialog is NULL\n",	EX_NULL_ERR);
		// parameters tree API
		CHECK_NULL(pFc->GpePosLib_CreatePrmTree,"iFace CreatePrmTree is NULL\n",EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_ReleaseTree,	"iFace ReleaseTree is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_GetInputTree,	"iFace GetInputTree is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_AddStringTag,	"iFace AddStringTag is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_AddIntTag,	"iFace AddIntTag is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_AddSubTree,	"iFace AddSubTree is NULL\n",	EX_NULL_ERR);	
		CHECK_NULL(pFc->GpePosLib_GetStringTag,	"iFace GetStringTag is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_GetIntTag,	"iFace GetIntTag is NULL\n",	EX_NULL_ERR);
		CHECK_NULL(pFc->GpePosLib_GetSubTree,   "iFace GetSubTree is NULL\n",	EX_NULL_ERR);

		printf("Loading OK\n");		// The base version API is loaded
		retLoad = 0;
		goto EX_FINALLY;

EX_NULL_ERR:
		retLoad = -1;
		printf("Fail in loading DLL functions\n");
    }
EX_FINALLY:
	return(retLoad);
}

int dynamicFreeLibrary(void)
{
    if(hDll != NULL)
        FreeLibrary(hDll);

    return(0);
}

/*****************************************************************************************
* Function test fod Dll and transaction over ECR- POS. Test starting first load dll, 
*  then make transaction and finish are unload dll
*****************************************************************************************/
static void loadTestDll(unsigned int cnt)
{
	int ret;
	unsigned int c;
	c = cnt; 
	while(c--)
	{
		// Unload dll
 		dynamicFreeLibrary();
		// Load Dll
		ret = dynamicLoadLibrary(&fcEcr, ECRDLLNAME);
		if(0 == ret)
		{
			if(fcEcr.GpePosLib_Open != NULL)
			{
			if(fcEcr.GpePosLib_Open("  ") == GPE_OK)
			{
				if(initEcrMain(&fcEcr) == 0)
                {
					std::cout << "The ECR started\n" << std::endl;
                    delay = 5;   // delay time betwing tested
					setLoadAmount(30);
					ret = callPurchase(&fcEcr, 1);
                }
                else
				{
                    printf("InitEcrMain failed");
					break;
				}
			}
			else
			{
                printf("PosLib Open Failed!\n");
				break;
			}
		}
        else
		{
            printf("PosLib not correctlly loaded!\n");
			break;
		}
	}
	else
	{
		std::cout << "Error load DLL " << std::endl;
		break;
	}
  }// end while()	
}
