/*
    +-----------------------------------------------------------------------+
    |   File name :             systemApi.cpp                               |
    |   Description :           Main test module for usage the DLL interface|
    |                                                                       |
    |                           Iterface between the ECR test base and DLL  |
    |                           library DLL.                                |    
    |                                                                       |
    |   Creation date :         27-05-2009                                  |
    |                                                                       |
    |   @author:                Pavel Perman                                | 
    |   @version:               1.0 (developed)                             |
    |   Copyright :             GPE                                         |
    |                                                                       |
    +-----------------------------------------------------------------------+
*/
#ifdef  WIN32
    #include <windows.h>
    #include <Mmsystem.h>
#else

#endif

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "Logger.h"
#include "SystemApi.h"

static int              csCount = 0;                    // pocet pouzitych kritickych sekci
static CRITICAL_SECTION criticalSec;                                   

/*
    Only test !!!!
*/
//static int getVersionMSOperatingSystem(char *pVersion)
//{
//    int ret = 0;
//    if(pVersion != NULL)
//        strcpy(pVersion, MS_WINDOWS_98);
//    else
//        ret = -1;
//
//    return(ret);
//}
//
//static int getVersionLinuxOperatingSystem(void)
//{
//
//}

/**
    Funkce vrati aktualni pocet ms od nastartovani PC.
*/
unsigned long SysApi_GetTimeInMs(void)
{
    unsigned long  retTime = 0;    
 
    #ifdef WIN32    
        
        MMTIME          pmmt;           
        
        if(timeGetSystemTime(&pmmt, sizeof(pmmt)) == TIMERR_NOERROR)
            retTime = 0;
        else
            retTime = (unsigned long) pmmt.u.ms;
    #endif
    
    return(retTime);
}

int SysApi_GetDateTime(SysApi_Time_t *pTime, SysApi_Date_t *pDate)
{
   time_t actTime;
    struct tm *timeinfo;

#ifdef WIN32    
//        SYSTEMTIME systemTime;
//        GetSystemTime(&systemTime);
///     TIME_ZONE_INFORMATION timeZone;
//        SYSTEMTIME localTime;
//                
//        GetTimeZoneInformation(&timeZone);                    
//
//        GetSystemTime(&systemTime);             // the returned time is in the UTC (Coordinated Universal Time)
//        SystemTimeToTzSpecificLocalTime(NULL, &systemTime, &localTime); // prepocet casu na casovou zonu. (NULL uvadi aktualni casovou zonu)
//
//        (pTime->hour)  = systemTime.wHour;
//        (pTime->min)   = systemTime.wMinute;
//        (pTime->sec)   = systemTime.wSecond;
//
//        (pDate->day)   = systemTime.wDay;
//        (pDate->month) = systemTime.wMonth;
//        (pDate->year)  = systemTime.wYear;
#endif

         actTime = time(0);    
         timeinfo = localtime(&actTime);        

        (pTime->hour)  = timeinfo->tm_hour;
        (pTime->min)   = timeinfo->tm_min;
        (pTime->sec)   = timeinfo->tm_sec;

        (pDate->day)   = timeinfo->tm_mday;
        (pDate->month) = timeinfo->tm_mon+1;
        (pDate->year)  = timeinfo->tm_year+1900;

    return(0);
}
 
/*
    
    (out) - pTimeStam: retezec znaku reprezentujici time stamp max [12 znaku]. Retezec je zakoncen nulou.

    Return: ( 0) - OK
            (-1) - neplatny vstupni parametr.       
            (-2) - interni chyba funkce (nepodarilo se ziskat systemovy cas)
*/
int SysApi_GetTimeStamp(char *pMask, char *pTimeStamp)
{
    int ret = 0;
    char year[5];
    SysApi_Time_t time;
    SysApi_Date_t date;
    
    if(pTimeStamp != NULL)
    {        
        if(SysApi_GetDateTime(&time, &date) == 0)
        { 
            sprintf(year, "%u", date.year);                             
            
            if(pMask != NULL)
                DBG_Output("stamp Len: %u", strlen(pMask));
            
            if((pMask != NULL) && (strlen(pMask) == TS_MASK_LEN))
                sprintf(pTimeStamp, "%c%s%c%02u%c%02u%c%02u%c%02u%c%02u%c", *pMask, &year[2],*(pMask+1), date.month,*(pMask+2), date.day,*(pMask+3), time.hour, *(pMask+4), time.min, *(pMask+5), time.sec, *(pMask+6));
            else
                sprintf(pTimeStamp, "%s%02u%02u%02u%02u%02u", &year[2], date.month, date.day, time.hour, time.min, time.sec);
        }
        else
            ret = -2;
    }
    else
        ret = -1;

    return(ret);
}

void SysApi_Clrscr(void)
{
    #ifdef WIN32        
        system("CLS");
    #else
        system("clear");
    #endif
} 

/*
            ( 0) CMD OK
    Return: (-1) CMD Fail
*/
int SysApi_ShellCmd(char const *pCmd)
{
    int ret = 0;
    if((pCmd != NULL) && (strlen(pCmd)>0))
    {
        #ifdef WIN32            
            if(system(pCmd) != 0)
                ret = -1;
        #else
            if(system(pCmd) == 0)       /* TODO: overit navratovou hodnotu na Linuxu? */
                ret = -1;
        #endif
    }
    return(ret);
}

/*
    (NULL) - fail in create
    (!=NULL) - OK
*/
SysApi_HANDLE SysApi_CreateThread(void *thProcedure, void *pParam)
{    
    SysApi_HANDLE rethandle = 0;
    
    #ifdef WIN32
        HANDLE      thHandle;        
        DWORD       threadId;                  
        
        thHandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)thProcedure, (LPVOID) pParam, 0, &threadId);
        
        if(thHandle != NULL)
            DBG_Output("Thread created %u\n", threadId);
        else
            DBG_Output("Fail in creating thread\n");
        
        rethandle = thHandle;
    #endif

    return(rethandle);
}

/*
 0 - fail
 !0 - OK(handle)
 ERROR_ALREADY_EXISTS - fail
*/
SysApi_HANDLE SysApi_CreateEvent(char *pNameEvent)
{
    SysApi_HANDLE rethandle = 0;

    #ifdef WIN32        
        rethandle = (SysApi_HANDLE) CreateEvent(NULL, TRUE, FALSE, (LPCTSTR) pNameEvent);
    #endif
    
    return(rethandle);
}

/*
   ( 1) - Object je v signalizovanem stavu
   ( 0) - TimeOut expired
   (-1) - TimeoutExpired
*/
int SysApi_WaitForSignal(SysApi_HANDLE syncEvent, unsigned long sysTimeout)
{
    int ret = 0;
    unsigned long retWait;

    #ifdef WIN32    
        DWORD timeout;

        if(sysTimeout == SYS_WAIT_INFINITE)
            timeout = INFINITE;
        else
            timeout = sysTimeout;

        retWait = (unsigned long) WaitForSingleObject((HANDLE)syncEvent, timeout);
        if(retWait == WAIT_OBJECT_0)
        {            
            ResetEvent(syncEvent);  
            ret = 1;
        }
        else if(retWait == WAIT_TIMEOUT)        
            ret = 0;
        else
            ret = -1;
        
    #endif

    return(ret);
}

/*
retrun: 0 - FAIL
        !0 - OK
*/
int SysApi_SendSignal(SysApi_HANDLE syncEvent)
{
    int ret;    
    
    #ifdef WIN32
        ret = SetEvent(syncEvent); 
    #endif

    return(ret);
}


/*
    NULL - error, next info from calling next last error.
    !0 (value is a handle to new mutex object)
*/
SysApi_HANDLE SysApi_CreateMutex(char * pMutexName)
{
    SysApi_HANDLE rethandle = 0;

    #ifdef WIN32        
        rethandle = (SysApi_HANDLE) CreateMutex(NULL, FALSE, (LPCTSTR) pMutexName);         //primarne bude mutex vlastnit caller process
    #endif
    
    return(rethandle);
}

/*
    NULL - error, next info from calling next last error.
    !0 (value is a handle to new mutex object)
*/
SysApi_HANDLE SysApi_Mutex(char * pMutexName)
{
    SysApi_HANDLE rethandle = 0;

    #ifdef WIN32
        rethandle = (SysApi_HANDLE) CreateMutex(NULL, FALSE, (LPCTSTR) pMutexName);         //primarne bude mutex vlastnit caller process
    #endif
    
    return(rethandle);
}


/*
    Vytvori synchronizacni objekt ke sdilenemu systemovemu zdroji, nebo ke sdilene promenne.
*/
SysApi_HANDLE SysApi_CreateLockObject(void)
{    
    #ifdef WIN32                
    
        //static CRITICAL_SECTION criticalSec[MAX_LOCK_NUMBER];        
        
        DBG_Output("Create critical section ...");
        InitializeCriticalSection(&criticalSec);
        csCount++;
        //return((SysApi_HANDLE) criticalSec);
        return(0);
    
    #endif
}

/*
    ( 0) - OK
    (-1) - Fail
*/
int SysApi_DeleteLockObject(SysApi_HANDLE pHandleCs)
{
    if(csCount)    
    {
        csCount--;
        DBG_Output("Delete critical section ...");
        //DeleteCriticalSection((CRITICAL_SECTION *)pHandleCs);    
        DeleteCriticalSection(&criticalSec);    
    }
    else
        DBG_Output("NO critical section ...");
    return(0);
}

int SysApi_UnlockObject(SysApi_HANDLE handleCs)
{
    if(csCount)
    {
        DBG_Output("Leave critical section ...");
//        LeaveCriticalSection((CRITICAL_SECTION *) handleCs);
        LeaveCriticalSection(&criticalSec);
    }
    else
        DBG_Output("NO critical section ...");

    return(0);
}

/*
    ( 0) - OK
    (-1) - Fail
*/
int SysApi_LockObject(SysApi_HANDLE handleCS)
{
    #ifdef WIN32        
        DBG_Output("Enter critical section ...");
        //EnterCriticalSection((CRITICAL_SECTION *) handleCS);
        EnterCriticalSection(&criticalSec);
    #endif
        
    return(0);
} 


/*
    0 - FALSE
    1 - TRUE
*/
int SysApi_CloseResource(SysApi_HANDLE *pCloseHandle)
{    
    int retHandle;
    
    #ifdef WIN32        
    
    if(*pCloseHandle != NULL)    
    {
        retHandle = (int) CloseHandle((HANDLE) *pCloseHandle);
        
        if(retHandle)
            *pCloseHandle = NULL;
    }
    #endif
    return(retHandle);
}


/*
  nothing
*/
void SysApi_Delay(unsigned int time_ms)
{
    Sleep(time_ms);
}


//TODO: odstranit
int SysApi_WaitGetKey(void)
{
    int x=0;
    int actKey = -1;

    do
    {
        x = getchar();
        if(x == 10 && actKey == -1)
            ;
        else
        {
            actKey = x;
            x = getchar();
            //if(x == 10 || x == 82)          // 'R' - 82
            if(x == 10)
                break;
        }
    }while(1);

    return(actKey);
}

